//---------------------------------------------
//  App: Summit Administration Application
//  (c) TASC Systems Inc. 2014-2018

//=============================================
// Summit Constants and Dependencies
//#region Summit Constants
var DIGITAL_INPUTS_BASE = 352;
var DIGITAL_INPUTS_POINTS = 48;

var DIGITAL_OUTPUTS_BASE = 496;
var DIGITAL_OUTPUTS_POINTS = 16;

var ANALOG_INPUTS_POINTS = 32;
var ANALOG_OUTPUTS_POINTS = 4;
var analogInputRangeDefault = "10";// Default range = 0 to 10V
var analogInputLowThreshold = "0";
var analogInputHighThreshold = "10";
var analogInputOffset = "0";

var SummitWebServerConfiguration = {};
var SUMMIT_HTTP_PORT = 80;
var SUMMIT_HTTPS_PORT = 443;

var ERRORLOG_FAILURE = 'FAILURE';
var ERRORLOG_WARNING = 'WARNING';
var ERRORLOG_NORMAL = 'NORMAL';

// Bootup order
var INITIALIZE_BASE_MODULES = 1000;
var INITIALIZE_IO_MODULES = 3000;
var INITIALIZE_SYSTEM_CORE = 7000;
var INITIALIZE_CLIENT_CONNECTIONS = 9000;

var PING_SENSOR_INTERVAL = 10// Default = 10seconds
var DIGITAL_INPUTS_CYCLE_TIME = 500// Default = 500ms (possible 400ms)
var DIGITAL_OUTPUTS_CYCLE_TIME = 500// Default = 400ms
var ANALOG_INPUTS_CYCLE_TIME = 750// Default = 750ms (possible 500ms)
var ANALOG_OUTPUTS_CYCLE_TIME = 500// Default = 200ms

// SNMP - this is here for now, could move
var AuthProtocol = ["None", "MD5", "SHA"];
var PrivProtocol = ["None", "DES", "AES"];

var DigitalInputsManager = null;
var DigitalOutputsManager = null;
var AnalogInputsManager = null;
var AnalogOutputsManager = null;
var SerialPortsManager = null;
var PingSensorManager;// Ping Manager Module

var SummitConfiguration = null;
var NetworkManager;// Network Manager module
var SnmpHostAddress;// SNMP Host address
var SmtpManager// SMTP Manager module
var SmtpMailer;// Email Module
var FactoryReset;// Factory Reset Module
var SummitStatusManager;// System Status Module
var SummitSecurity;// Summit Security/Users Module
var SummitUpgradeManager;// Summit database upgrade module
var SummitInfo = { "MACID": "1C:87:79:70:xx:xx", "Name": "Summit" };// Default Summit information
var SummitConfigurationModule;// Global structure holding Summit configuration

var SnmpManager = [];// Global array of SNMP Hosts
var PingSensorRules = [];// Global array of Ping Sensor Control Rules

//#endregion Summit Constants

//=============================================
//  Crest Infrastructure Initialization
//#region Crest Infrastructure
// Initializing Logging
//#region Logging Infrastructure
// Start the logger
var log4js = require('log4js');

//config
log4js.configure({
    appenders: [
        //{ type: 'console' },
        {
            type: "file",
            absolute: true,
            filename: "./log/summit.log",
            maxLogSize: 1000000,
            backups: 10
        }
    ]
});
var LogManager = log4js.getLogger();
LogManager.setLevel(log4js.levels.INFO);// Toggle to this if you want just basic log info
//LogManager.setLevel(log4js.levels.DEBUG);// Toggle to this if you want all log info
//LogManager.setLevel(log4js.levels.ALL);// Toggle to this if you want all log info
//#endregion Logging Infrastructure

// Initializing server
//#region Server Infrastructure

//#region Startup log
var os = require('os');
var crestJson = require('./package.json')
LogManager.info('============================================================================================');
LogManager.info(crestJson.description);
LogManager.info('   Version ' + crestJson.version + ' (' + crestJson.date + ') ')
LogManager.info('   (c) ' + crestJson.author);
//logger.info('   Serial Number [coming soon: serial number of this device]');
LogManager.info('   ' + os.platform() + " OS Version " + os.release())
LogManager.info('   ' + 'Node Version ' + process.version);
LogManager.info('   ' + os.freemem() + ' Free / ' + os.totalmem() + ' Total Memory');
LogManager.info('--------------------------------------------------------------------------------------------');
//#endregion Startup log

LogManager.debug('Intializing server infrastructure')

//---------------------------------------------
// Initializing Summit users and security
//#region Summit Users
LogManager.info('Intializing Summit user management');
SummitSecurity = require('./users.js');
SummitSecurity.Initialize(LogManager);
var summitSecurityOptions = SummitSecurity.ReadSummitSecurityOptions();
//#endregion Summit Users

// Fire up the server + socket connections
var express = require('express');
var app = express();

// For serving static pages
var serveStatic = require('serve-static');

// For parsing middleware - e.g., JSON body
var bodyparser = require('body-parser');
app.use(bodyparser.json());
app.use(bodyparser.urlencoded({ extended: true }));

// For server and filesystem
var fs = require('fs');

var http = require('http').Server(app);
var https = require('https').Server(summitSecurityOptions, app);
var cp = require('child_process');
var path = require('path');
var moment = require('moment');
var CrestStartTime = moment();

// For socket.io messaging
//#region socket.io messaging
var ClientConnectedCounter = 0;
var ioServer = require('socket.io');
SocketIoManager = new ioServer();
SocketIoManager.attach(http, { 'transports': ['polling', 'websocket'], 'allowUpgrades': true, "pingInterval": 12500, "pingTimeout": 30000 });
SocketIoManager.attach(https, { 'transports': ['polling', 'websocket'], 'allowUpgrades': true, "pingInterval": 12500, "pingTimeout": 30000 });
SocketIoManager.sockets.on('connection', function (socket) {
    ClientConnectedCounter++;
    var address = socket.handshake.address;
    LogManager.warn('socket.connection=' + socket.id + ' client=' + address + ' (# clients=' + ClientConnectedCounter + ')');

    socket.on('reconnect', function () {
        LogManager.warn('socket.reconnection=' + socket.id + ' client=' + address + ' (# clients=' + ClientConnectedCounter + ')');
    });

    socket.on('disconnect', function (reason) {
        ClientConnectedCounter--;
        LogManager.warn('socket.disconnect=' + reason + ' id=' + socket.id + ' client=' + address + ' (# clients=' + ClientConnectedCounter + ')');
    });

    socket.on('error', function (error) {
        LogManager.error('socket.error=' + error + ' id=' + socket.id + ' client=' + address);
    });
});
//#endregion socket.io messaging
//#endregion Server Infrastructure
//#endregion Crest Infrastructure

//=============================================
// Crest Database
//#region Crest Database
//  Here we initialize the database and 
//  make any upgrades before we move on
//  to starting the core Crest system

// Initializing database infrastucture
//#region Database Infrastructure
LogManager.info('Intializing Summit database')
var sqlite3 = require('sqlite3').verbose();
var SqliteManager = new sqlite3.Database('data/summit.db');
//#endregion Database Infrastructure

// Upgrade database if necessary
//#region Database Infrastructure
SummitUpgradeManager = require('./upgrade.js');
SummitUpgradeManager.UpgradeDatabase(crestJson, SqliteManager, LogManager);
//#endregion Database Infrastructure
//#endregion Crest Database

//=============================================
// Crest Base Modules Initialization
//#region Crest Base Modules Initialization
//  The following initialization of the system
//  waits for the database to be initialized
//  and upgraded before other modules which may
//  be dependent on the database are launched. 
setTimeout(function () {
    // Initialize System Status Manager
    //#region System Status Manager
    LogManager.warn('Crest Bootup: 1. Initialize Base Modules Start')
    SummitStatusManager = require('./systemstatus.js')
    SummitStatusManager.InitializeSystemStatusManager(SqliteManager, LogManager, function () {
        SummitStatusManager.LogSystemStatus(crestJson.description + " Version " + crestJson.version + " started", ERRORLOG_NORMAL, fs, SocketIoManager, LogManager, SqliteManager);
        SummitStatusManager.EnsureFactoryResetInterfacesFile(LogManager);
    });
    //#endregion System Status Manager

    // Initializing network management
    //#region Network Management
    LogManager.info('Intializing Summit Network and SNMP objects')
    NetworkManager = require('./networkmanager.js');
    SmtpMailer = require('nodemailer');// SMTP support
    NetworkManager.InitializeNetworkInterfaces(SqliteManager, LogManager);
    NetworkManager.InitializeSnmpHostSettings(function (snmpHostsArray) {
        SnmpManager = snmpHostsArray;// Initialize the global SNMP Hosts array
    });

    SmtpManager = require('./smtp.js');
    SmtpManager.InitializeSmtp(SmtpMailer, LogManager, function (smtp) {
        // Do nothing for now ... maybe later, maybe never
    });
    //#endregion Network Management

    // Initialize the IO modules
    //#region IO Modules
    DigitalInputsManager = require('./digitalinputs.js');
    DigitalOutputsManager = require('./digitaloutputs.js');
    AnalogInputsManager = require('./analoginputs.js');
    AnalogOutputsManager = require('./analogoutputs.js');
    SerialPortsManager = require('./serialports.js');
    //#endregion IO Modules

    // Initializing Ping Sensor Manager
    //#region Ping Sensor Manager
    LogManager.info('Intializing Ping Sensor Manager')
    PingSensorManager = require('./kenwood.js')
    //#endregion Ping Sensor Manager

    // Initializing Factory Reset
    //#region Factory Reset
    LogManager.info('Intializing Factory Reset')

    FactoryReset = require('./factoryreset.js');
    //#endregion Factory Reset

    LogManager.warn('Crest Bootup: 1. Initialize Base Modules Done')
}, INITIALIZE_BASE_MODULES);// e.g., after 1 periods
//#endregion Crest Base Modules Initialization

//=============================================
// Crest IO Module Initialization
//#region Crest IO Module Initialization
setTimeout(function () {
    LogManager.warn('Crest Bootup: 2. Initialize I/O Modules Start')

    // Read the Summit Configuration, needed to set the Voltage formulae
    SummitConfigurationModule = require("./summitconfig.js");
    // Get the network interfaces - so we can set the SummitInfo=MACID
    NetworkManager.GetNetworkInterfaces(function (err, networkInterfaces) {
        if (err) {
            LogManager.error('Initializing Summit Network: Error retrieving Network Interfaces. ' + err);
        } else {
            LogManager.info('Initializing Summit Network: : Retreived Network Interfaces=' + JSON.stringify(networkInterfaces));
            for (var i = 0; i < networkInterfaces.length; i++) {
                if (networkInterfaces[i].Name === "eth0") {
                    SummitInfo = { "MACID": networkInterfaces[i].MACID, "Name": networkInterfaces[i].MACID };
                    break;
                }
            }
            LogManager.info("Initialize Summit Configuration: Initializing SummitInfo to =" + JSON.stringify(SummitInfo));
        }

        SummitConfigurationModule.Initialize(LogManager, SummitInfo, function (summitInfo, summitConfiguration) {
            SummitInfo = summitInfo;

            SummitConfiguration = summitConfiguration;

            // Initialize the Analog Formulae
            SummitConfigurationModule.GetAnalogVoltageFormulae(SummitConfiguration, function (analogVoltageFormulae) {
                AnalogInputsManager.InitializeVoltageFormula(analogVoltageFormulae.inputs, LogManager);// Need to pass logger here
                AnalogOutputsManager.InitializeVoltageFormula(analogVoltageFormulae.outputs, LogManager);// Need to pass logger here

                LogManager.warn('Crest Bootup: 2. Initialize I/O Modules Done')
            });
        });
    });
}, INITIALIZE_IO_MODULES);// e.g., after 2 periods
//#endregion Crest IO Module Initialization

//=============================================
// System Core
//#region System Core
//  Don't start up the core until all of the 
//  system modules are initialized because
//  the core is dependent on these modules.
setTimeout(function () {
    LogManager.warn('Crest Bootup: 3. Initialize I/O Core Start')

    LogManager.info("System Core: Initialize the analog input formulas");
    AnalogInputsManager.InitializeFormulas(SqliteManager, LogManager, SocketIoManager, function () {
        LogManager.info("System Core: Analog Input formulas are initialized");
    });

    //---------------------------------------------
    //  Factory reset processing starts here....
    LogManager.info("System Core: Factory Reset Processing starting")
    FactoryReset.ProcessFactoryReset(SqliteManager, LogManager, SocketIoManager, SummitStatusManager, function () {

    });

    //---------------------------------------------
    //  System I/O processing starts here....
    LogManager.info("System Core: I/O Processing starting")

    DigitalInputsManager.InitializeDigitalInputs(SummitInfo, SmtpManager, SnmpManager, SqliteManager, SocketIoManager, LogManager, function () {
        LogManager.info('System Core IO: Digital Inputs initialized and being processed (' + DIGITAL_INPUTS_CYCLE_TIME + 'ms)')
        setInterval(processDigitalInputs, DIGITAL_INPUTS_CYCLE_TIME);
        function processDigitalInputs() {
            DigitalInputsManager.ProcessDigitalInputs(SummitStatusManager, function (digitalInputsArray) {
                // After reading the digital input values, update the analog input qualifier values
                AnalogInputsManager.UpdateQualifierValues(digitalInputsArray)
                SerialPortsManager.UpdateSerialPortQualifiers(digitalInputsArray);
            })
        }
    });

    DigitalOutputsManager.InitializeDigitalOutputs(SummitInfo, SmtpManager, SnmpManager, SqliteManager, SocketIoManager, LogManager, function () {
        LogManager.info('System Core IO: Digital Outputs initialized')
        DigitalOutputsManager.ResetDigitalOutputs(SummitStatusManager, function () {
            LogManager.info('System Core IO: Digital Outputs being processed (' + DIGITAL_OUTPUTS_CYCLE_TIME + 'ms)')
            setInterval(processDigitalOutputs, DIGITAL_OUTPUTS_CYCLE_TIME);
            function processDigitalOutputs() {
                DigitalOutputsManager.ProcessDigitalOutputs(SummitStatusManager)
            }

            // Since the Ping Sensor Manager is dependent on digital outputs, we need to have them started first
            LogManager.info('System Core IO: Start Ping Sensor Manager');
            PingSensorManager.InitializeKenwoodManager(SummitInfo, SmtpManager, SnmpManager, SqliteManager, SocketIoManager, LogManager, function (InitializedKenwoodControlArray) {
                PingSensorRules = InitializedKenwoodControlArray.slice();
                LogManager.info("System Core: Initialized Ping Sensor Rules=" + JSON.stringify(PingSensorRules));
                LogManager.info('System Core: Ping Sensors being processed (' + PING_SENSOR_INTERVAL * 1000 + 'ms)')
                LogManager.warn('Crest Bootup: 3. Initialize I/O Core Done')

                setTimeout(processPingSensors, PING_SENSOR_INTERVAL * 1000);
                function processPingSensors() {
                    PingSensorManager.ProcessKenwoodPings(SummitStatusManager, DigitalOutputsManager, function () {
                        // After pinging the host(s), do something if we need to ...
                        //logger.info("SystemCore: Completed Processing of Ping Sensor, reset Timeout to: "+ PING_SENSOR_INTERVAL * 1000)
                        setTimeout(processPingSensors, PING_SENSOR_INTERVAL * 1000)
                    });
                }
            });
        });
    });

    AnalogInputsManager.InitializeAnalogInputs(SummitInfo, SmtpManager, SnmpManager, SqliteManager, SocketIoManager, LogManager, function () {
        LogManager.info('System Core IO: Analog Inputs initialized and being processed (' + ANALOG_INPUTS_CYCLE_TIME + 'ms)')
        setInterval(ProcessAnalogInputs, ANALOG_INPUTS_CYCLE_TIME);
        function ProcessAnalogInputs() {
            AnalogInputsManager.ProcessAnalogInputs(SummitStatusManager)
        }
    });

    AnalogOutputsManager.InitializeAnalogOutputs(SummitInfo, SmtpManager, SnmpManager, SqliteManager, SocketIoManager, LogManager, function () {
        LogManager.info('System Core IO: Analog Outputs initialized')
        AnalogOutputsManager.ResetAnalogOutputs(SummitStatusManager, function () {
            LogManager.info('System Core IO: Analog Outputs reset and being processed (' + ANALOG_OUTPUTS_CYCLE_TIME + 'ms)')
            setInterval(processAnalogOutputs, ANALOG_OUTPUTS_CYCLE_TIME);
            function processAnalogOutputs() {
                AnalogOutputsManager.ProcessAnalogOutputs(SummitStatusManager)
            }
        });
    });

    SerialPortsManager.InitializeSerialPorts(SummitInfo, SmtpManager, SnmpManager, SqliteManager, SocketIoManager, LogManager, function () {
        LogManager.info('System Core IO: Serial Ports initialized')
    });

}, INITIALIZE_SYSTEM_CORE);// e.g., after 3 periods
//#endregion System Core

//=============================================
// Crest Client Management
//#region Crest Client Management
//  Don't start client connections until 
//  system core is up and running, we don't
//  want anyone accessing the system until
//  it is fully up and running
setTimeout(function () {
    LogManager.warn('Crest Bootup: 4. Initialize Client Connections Manager Start')

    SummitSecurity.GetWebServerConfiguration(function (summitWebServerConfiguration) {
        SummitWebServerConfiguration = summitWebServerConfiguration;
        LogManager.info("Initializing Web Server Configuration=" + JSON.stringify(SummitWebServerConfiguration));
        SUMMIT_HTTP_PORT = SummitWebServerConfiguration.http.port;
        SUMMIT_HTTPS_PORT = SummitWebServerConfiguration.https.port;

        // http being enabled is dependent on configuration
        if (SummitWebServerConfiguration.http.enabled) {
            http.listen(SUMMIT_HTTP_PORT, function () {
                LogManager.info('HTTP web server listening on port ' + SUMMIT_HTTP_PORT)
            });
        }

        // But https is always enabled - can't be disabled!
        https.listen(SUMMIT_HTTPS_PORT, function () {
            LogManager.info('HTTPS web server listening on port ' + SUMMIT_HTTPS_PORT)
            LogManager.warn('Crest Bootup: 4. Initialize Client Connections Manager Done')
        });
    });
}, INITIALIZE_CLIENT_CONNECTIONS);// e.g., after 4 periods
//#endregion Crest Client Management

//=============================================
// Crest API Middleware
//  All of the REST interface is found here
//#region Crest API Middleware 

// Initializing Crest API Utility Functions
//#region API Middleware Initialization
LogManager.info('Initializing Summit API')
app.use(serveStatic('.', { 'index': ['index.html'] }))
app.use(serveStatic('views', { 'index': false }))

app.use(function (req, resp, next) {
    if (req.headers['x-forwarded-proto'] == 'http') {
        return resp.redirect(301, 'https://' + req.headers.host + '/');
    } else {
        return next();
    }
});
//#endregion API Middleware Initialization

//#region API Utility Functions
app.get('/api/causeerror', function (req, res, next) {
    LogManager.info('Summit API: GET /api/causeerror');
    res.send('caused error');
    process.exit(-1);
})
//#endregion API Utility Functions

//#region Summit Web Server
// Get Summit Web Server Configuration
app.get('/api/webserver', function (req, res, next) {
    LogManager.info('Summit API: GET /api/webserver');
    SummitSecurity.GetWebServerConfiguration(function (summitWebServerConfiguration) {
        LogManager.info("Summit API: Summit Web Server Configuration=" + JSON.stringify(summitWebServerConfiguration));
        res.send(summitWebServerConfiguration);
    });
});

// Put Summit Web Server Configuration
app.put('/api/webserver', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/webserver');
    SummitSecurity.PutWebServerConfiguration(req.body, function (summitWebServerConfiguration) {
        LogManager.info("Summit API: PUT Summit Web Server Configuration=" + JSON.stringify(summitWebServerConfiguration));
        res.send(summitWebServerConfiguration);
    });
});

//#endregion Summit Web Serve

//#region Summit Authentication

// Login Summit User
//  Function for both local or RADIUS server authentication
//  LoginResponse: Result (true if OK, false if failed), Message (explanation)
app.put('/api/login', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/login');
    var ip = req.headers['x-real-ip'] || req.connection.remoteAddress
    SummitSecurity.LoginUser(req.body, ip, function (LoginResponse) {
        LogManager.info('Summit API: Login response=' + JSON.stringify(LoginResponse));
        res.send(LoginResponse);
    });
});

// Get Summit Login Attempts
app.get('/api/login/attempts', function (req, res, next) {
    LogManager.info('Summit API: GET /api/login/attempts');
    SummitSecurity.GetSummitLoginAttempts(function (SummitLoginAttempts) {
        LogManager.info("Summit API: Summit Login Attempts=" + SummitLoginAttempts.length);
        res.send(SummitLoginAttempts);
    });
});

// Get Summit Authentication Configuration
app.get('/api/login/authconfig', function (req, res, next) {
    LogManager.info('Summit API: GET /api/login/authconfig');
    SummitSecurity.GetAuthenticationConfiguration(function (summitAuthenticationConfiguration) {
        LogManager.info("Summit API: Summit Authentication Configuration=" + JSON.stringify(summitAuthenticationConfiguration));
        res.send(summitAuthenticationConfiguration);
    });
});

// Put Summit Authentication Configuration
app.put('/api/login/authconfig', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/login/authconfig');
    SummitSecurity.PutAuthenticationConfiguration(req.body, function (summitAuthenticationConfiguration) {
        LogManager.info("Summit API: PUT Summit Authentication Configuration=" + JSON.stringify(summitAuthenticationConfiguration));
        res.send(summitAuthenticationConfiguration);
    });
});

// Get Summit Local Authentication Configuration
app.get('/api/login/localconfig', function (req, res, next) {
    LogManager.info('Summit API: GET /api/login/localconfig');
    SummitSecurity.GetLocalAuthenticationConfiguration(function (summitLocalAuthenticationConfiguration) {
        LogManager.info("Summit API: Get Summit Local Authentication Configuration [User=" + summitLocalAuthenticationConfiguration.user + "]");
        res.send(summitLocalAuthenticationConfiguration);
    });
});

// Put Summit Local Authentication Configuration
app.put('/api/login/localconfig', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/login/localconfig');
    SummitSecurity.PutLocalAuthenticationConfiguration(req.body, function (summitLocalAuthenticationConfiguration) {
        LogManager.info("Summit API: PUT Summit Local Authentication Configuration [User=" + summitLocalAuthenticationConfiguration.user + "]");
        res.send(summitLocalAuthenticationConfiguration);
    });
});

//#endregion Summit Authentication

//#region Summit System API
//---------------------------------------------
// Summit System API

// Identify system
app.get('/api/summit/identify', function (req, res, next) {
    LogManager.info('Summit API: GET /api/summit/identify');
    // Flash lights yellow 1s, then blue 1s for a total of 1 minute
    var timeDelay = 3000;
    for (var i = 0; i < 5; i++) {
        setTimeout(function () {
            SummitStatusManager.WarningOperation(fs, SocketIoManager, LogManager);
        }, timeDelay * i);

        setTimeout(function () {
            SummitStatusManager.NormalOperation(fs, SocketIoManager, LogManager);
        }, timeDelay * i + (timeDelay / 2));
    }
    res.send('Identify Device OK');
})

// Get installed system software
app.get('/api/summit/operatingsystem', function (req, res, next) {
    LogManager.info('Summit API: GET /api/summit/operatingsystem');
    data = { "os": os.platform() + " OS Version " + os.release() };
    LogManager.debug('Summit API: GET /api/summit/operatingsystem - System OS=' + JSON.stringify(data))
    res.send(data);
})

// Put System Date/Time
app.put('/api/summit/systemdatetime', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/summit/systemdatetime ' + JSON.stringify(req.body));
    var exec = cp.exec;
    var ret;
    exec('date -s "' + req.body.SystemDateTime + '"', function (error, stdout, stderr) {
        if (error) res.send({ "data": "ERROR: " + error });
        else {
            res.send({ "data": "OK: " + stdout });
        }
    });
})

// Get current system status
app.get('/api/summit/systemstatus', function (req, res, next) {
    LogManager.info('Summit API: GET /api/summit/systemstatus');
    var CurrentSystemStatus = SummitStatusManager.GetCurrentSystemStatus(LogManager);
    LogManager.debug('Summit API: GET /api/summit/systemstatus - Current System Status=' + CurrentSystemStatus);
    res.send(CurrentSystemStatus);
})

// Get list of system events
app.get('/api/summit/systemevents', function (req, res, next) {
    LogManager.info('Summit API: GET /api/summit/systemstatus');
    var systemStatus = [];
    SqliteManager.all("SELECT ErrorMessage, ErrorType, DATE(TimeChanged) FROM ErrorLog ORDER BY DATE(TimeChanged) DESC LIMIT 20", function (err, rows) {
        if (err) {
            throw LogManager.error("Summit API: GET /api/summit/systemstatus - ERROR=" + err);
        }
        for (i = 0; i < rows.length; i++) {
            // Always send the raw value - clients responsible for conversion
            systemStatus.push({
                "ErrorType": rows[i].ErrorType,
                "ErrorMessage": rows[i].ErrorMessage,
                "TimeChanged": rows[i].TimeChanged
            });
        }
        LogManager.debug('Summit API: GET /api/summit/systemstatus - System Status=' + JSON.stringify(systemStatus));
        res.json(systemStatus);
    });
})

// Get Summit Info
app.get('/api/summit/info', function (req, res, next) {
    LogManager.info('Summit API: GET /api/summit/info = ' + JSON.stringify(SummitInfo));
    res.send(SummitInfo);
})

// Put Summit Info
app.put('/api/summit/info', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/summit/name ' + JSON.stringify(req.body));
    SummitConfigurationModule.WriteSummitInfo(req.body, function (result) {
        LogManager.debug('Summit API: GET /api/summit/info - Result=' + JSON.stringify(result));
        UpdateSummitInfo(result);
        res.send(result);
    });
})

function UpdateSummitInfo(info) {
    SummitInfo = info;
    DigitalInputsManager.UpdateSummitInfo(SummitInfo);
    DigitalOutputsManager.UpdateSummitInfo(SummitInfo);
    AnalogInputsManager.UpdateSummitInfo(SummitInfo);
    AnalogOutputsManager.UpdateSummitInfo(SummitInfo);
    SerialPortsManager.UpdateSummitInfo(SummitInfo);
    PingSensorManager.UpdateSummitInfo(SummitInfo);
}

// Get current Summit system configuration
app.get('/api/summit/systemconfig', function (req, res, next) {
    LogManager.info('Summit API: GET /api/summit/systemconfig = ' + JSON.stringify(SummitConfiguration));
    res.send(SummitConfiguration);
})
//#endregion

//#region Crest Software
// Get Crest software info
app.get('/api/crest', function (req, res, next) {
    LogManager.info('Summit API: GET /api/crest');
    LogManager.debug('Summit API: GET /api/crest - Crest info=' + JSON.stringify(crestJson))
    res.send(crestJson);
})

app.get('/api/crest/latest', function (req, res, next) {
    LogManager.info('Summit API: GET /api/crest/latest');
    SummitStatusManager.GetLatestCrestVersionFromServer(LogManager, function (latestversion) {
        LogManager.debug('Summit API: GET /api/crest/latest - Crest latest version=' + latestversion)
        res.send(latestversion);
    });
})

app.get('/api/crest/starttime', function (req, res, next) {
    LogManager.info('Summit API: GET /api/crest/starttime');
    res.send({ "CrestStartTime": CrestStartTime });
})

app.put('/api/crest/download', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/crest/download ' + JSON.stringify(req.body));
    SummitStatusManager.UpgradeCrestAfterDownload(req.body.CrestUrl, LogManager, function (result) {
        LogManager.info('Summit API: GET /api/crest/download - COMPLETE! - Crest downloaded and installed=' + result);
        res.send(result);
    });
})

var formidable = require('formidable');
app.post('/api/crest/upload', function (req, res, next) {
    LogManager.info('Summit API: POST /api/crest/upload');

    var form = new formidable.IncomingForm();
    form.uploadDir = ".";//set upload directory
    form.keepExtensions = true;     //keep file extension

    form.parse(req, function (err, fields, files) {
        res.writeHead(200, { 'content-type': 'text/plain' });
        res.write('received upload:\n\n');
        LogManager.info("Summit API: POST /api/crest/upload - bytesReceived=" + form.bytesReceived);
        //TESTING
        LogManager.info("Summit API: POST /api/crest/upload - Crest.tar size: " + files.CrestUpgradeFile.size);
        LogManager.info("Summit API: POST /api/crest/upload - Crest tar name: '" + files.CrestUpgradeFile.name + "'");
        LogManager.info("Summit API: POST /api/crest/upload - Last modified: " + files.CrestUpgradeFile.lastModifiedDate);

        //Formidable changes the name of the uploaded file
        //Rename the file to its original name
        fs.rename(files.CrestUpgradeFile.path, files.CrestUpgradeFile.name, function (err) {
            if (!err) {
                // Crest.tar file has been uploaded, so now let's do the upgrade!
                SummitStatusManager.UpgradeCrestAfterUpload(files.CrestUpgradeFile.name, LogManager, function (result) {
                    LogManager.info('Summit API: POST /api/crest/upload: COMPLETE! - Crest uploaded and installed=' + result);
                });
            }
        });
        res.end();
    });

    ////Formidable uploads to operating systems tmp dir by default
})

//#endregion Crest Software

//#region Networking, SMTP and SNMP API
//---------------------------------------------
// Networking, SMTP and SNMP

// Networking Support
//#region Networking Support
// Get network interfaces
app.get('/api/network/interfaces', function (req, res, next) {

    // Get Network Interfaces information from the Network Manager module
    LogManager.info('Summit API: GET /api/network/interfaces - Get Network Interfaces called');
    NetworkManager.GetNetworkInterfaces(function (err, networkInterfaces) {
        if (err) {
            LogManager.info('Summit API: GET /api/network/interfaces - Return Network Interfaces. Error=' + err);
            res.json({});
        } else {
            // Return the network interfaces
            LogManager.info('Summit API: GET /api/network/interfaces - Return Network Interfaces=' + JSON.stringify(networkInterfaces));
            res.json(networkInterfaces);
        }
    });

})

// Set network interfaces
app.put('/api/network/interfaces', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/network/interfaces - Updating Network Interface=' + JSON.stringify(req.body));
    NetworkManager.PutNetworkInterfaces(req.body, function (error, interfaces) {
        if (error) {
            LogManager.error('Summit API: PUT /api/network/interfaces - Updated Network Interface. Error=' + error);
        } else {
            LogManager.info('Summit API: PUT /api/network/interfaces - Updated Network Interface=' + JSON.stringify(interfaces));
        }
        res.send(interfaces);
    });
})
//#endregion Networking Support

// SMTP Support
//#region SMTP Support

// Get SMTP Settings
app.get('/api/network/smtp', function (req, res, next) {
    LogManager.info('Summit API: GET /api/network/smtp');
    SmtpManager.GetSmtpSettings(function (smtp) {
        LogManager.info('Summit API: GET /api/network/smtp - SMTP Settings=' + smtp);
        res.json(smtp);
    })
});

// Put SMTP Settings
app.put('/api/network/smtp', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/network/smtp');
    req.body.Default = false;// No longer using default value
    SmtpManager.SetSmtpSettings(req.body, function (smtp) {
        LogManager.info('Summit API: PUT /api/network/smtp - SMTP Settings=' + smtp);
        res.json(smtp);
    })
});

// Test a SMTP host 
app.put('/api/network/smtp/test', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/network/smtp/test - data=' + JSON.stringify(req.body));

    // create reusable transporter object using the default SMTP transport
    var transporter = SmtpMailer.createTransport({
        port: req.body.Port,
        host: req.body.Host,
        secure: req.body.Secure,
        auth: {
            user: req.body.User,
            pass: req.body.Pass
        }
    });

    // setup e-mail data with unicode symbols
    var mailOptions = {
        from: req.body.From, // sender address
        to: req.body.To, // list of receivers
        subject: 'Crest SMTP Test Email', // Subject line
        text: 'Your SMTP configuration is OK and so Crest is able to send you emails!', // plaintext body
    };

    // send mail with defined transport object
    var result = false;
    transporter.sendMail(mailOptions, function (error, info) {
        if (error) {
            LogManager.error("Summit API: PUT /api/network/smtp/test - send email error=" + error);
            result = false;
            res.json({ "result": result, "response": error });
        } else {
            LogManager.info("Summit API: PUT /api/network/smtp/test - send email result=" + info.response);
            result = true;
            res.json({ "result": result, "response": info.response });
        }
    });
});

//#endregion SMTP Support

// SNMP Support
//#region SNMP Support
// Get SNMP hosts
app.get('/api/network/snmp/hosts', function (req, res, next) {
    // Get the complete list of SNMP Hosts and update I/O processing sub-systems
    LogManager.info('Summit API: GET /api/network/snmp/hosts');

    var snmpHosts = [];
    SqliteManager.all("SELECT rowid as Id, HostName, Enabled, IpAddress, Port, Community, SnmpVersion, AuthUser, AuthPassword, AuthProtocol, PrivPassword, PrivProtocol FROM snmphosts ORDER BY Id", function (err, rows) {
        if (err) {
            // LogSystemStatus(errorMessage, errorType, timeStamp, db, systemStatusManager)
            throw LogManager.error('Summit API: GET /api/network/snmp/hosts - Error=' + err);
        }
        for (i = 0; i < rows.length; i++) {
            // Always send the raw value - clients responsible for conversion
            snmpHosts.push({
                "Id": rows[i].Id,
                "HostName": rows[i].HostName,
                "Enabled": rows[i].Enabled,
                "IpAddress": rows[i].IpAddress,
                "Port": rows[i].Port,
                "Community": rows[i].Community,
                "SnmpVersion": rows[i].SnmpVersion,
                "AuthUser": rows[i].AuthUser,
                "AuthPassword": rows[i].AuthPassword,
                "AuthProtocol": rows[i].AuthProtocol,
                "PrivPassword": rows[i].PrivPassword,
                "PrivProtocol": rows[i].PrivProtocol
            });
        }

        LogManager.debug('Summit API: GET /api/network/snmp/hosts - Hosts=' + JSON.stringify(snmpHosts));
        res.json(snmpHosts);
    });
})

// Delete a SNMP host
app.put('/api/network/snmp/hosts/delete', function (req, res, next) {
    // Update the existing record
    LogManager.debug('Summit API: PUT /api/network/snmp/hosts/delete - Host to delete=' + JSON.stringify(req.body));

    var update = "DELETE FROM SnmpHosts WHERE rowid='" + req.body.Id + "'";
    SqliteManager.run(update, function (err) {
        if (err !== null) {
            LogManager.error('Summit API: PUT /api/network/snmp/hosts/delete - Error (Update)=' + JSON.stringify(err));
            res.send({ msg: "SQLite3 dB: api/network/snmp/hosts/" + " DELETE FROM  err=" + JSON.stringify(err) })
        }

        // Get the complete list of SNMP Hosts and update I/O processing sub-systems
        var snmpHosts = [];
        SqliteManager.all("SELECT rowid as Id, HostName, Enabled, IpAddress, Port, Community, SnmpVersion, AuthUser, AuthPassword, AuthProtocol, PrivPassword, PrivProtocol FROM snmphosts ORDER BY Id", function (err, rows) {
            if (err) {
                throw LogManager.error('Summit API: PUT /api/network/snmp/hosts/delete - Error (Read)=' + JSON.stringify(err));
            }
            for (i = 0; i < rows.length; i++) {
                // Always send the raw value - clients responsible for conversion
                snmpHosts.push({
                    "Id": rows[i].Id,
                    "HostName": rows[i].HostName,
                    "Enabled": rows[i].Enabled,
                    "IpAddress": rows[i].IpAddress,
                    "Port": rows[i].Port,
                    "Community": rows[i].Community,
                    "SnmpVersion": rows[i].SnmpVersion,
                    "AuthUser": rows[i].AuthUser,
                    "AuthPassword": rows[i].AuthPassword,
                    "AuthProtocol": rows[i].AuthProtocol,
                    "PrivPassword": rows[i].PrivPassword,
                    "PrivProtocol": rows[i].PrivProtocol
                });
            }
            LogManager.debug('Summit API: PUT /api/network/snmp/hosts/delete - Hosts=' + JSON.stringify(snmpHosts));
            UpdateSnmpManager(snmpHosts);
            res.json(snmpHosts);
        });
    });
})

// Create new SNMP host
app.put('/api/network/snmp/hosts', function (req, res, next) {
    // Add a new record
    LogManager.debug('Summit API: PUT /api/network/snmp/hosts - Host to add=' + JSON.stringify(req.body));

    // Insert the new record
    SqliteManager.run("BEGIN")
    insertStatement = "'" + req.body.HostName + "', '" +
        req.body.Enabled + "', '" +
        req.body.IpAddress + "', '" +
        req.body.Port + "', '" +
        req.body.Community + "', " +
        req.body.SnmpVersion + ", '" +
        req.body.AuthUser + "', '" +
        req.body.AuthPassword + "', " +
        req.body.AuthProtocol + ", '" +
        req.body.PrivPassword + "', " +
        req.body.PrivProtocol;
    SqliteManager.run("INSERT INTO SnmpHosts (HostName, Enabled, IpAddress, Port, Community, SnmpVersion, AuthUser, AuthPassword, AuthProtocol, PrivPassword, PrivProtocol) VALUES (" + insertStatement + ")", function (err) {
        if (err !== null) {
            LogManager.error('Summit API: PUT /api/network/snmp/hosts - Error (Insert)=' + err);
            res.send({ msg: "SQLite3 dB: api/snmphosts Insert err=" + JSON.stringify(err) })
        }

        // Successful insert, just look up the ID for future reference
        LogManager.debug('Summit API: PUT /api/network/snmp/hosts - ID=' + this.lastID);
        req.body.Id = this.lastID;
    });
    SqliteManager.run("COMMIT");

    // Get the complete list of SNMP Hosts and update I/O processing sub-systems
    var snmpHosts = [];
    SqliteManager.all("SELECT rowid as Id, HostName, Enabled, IpAddress, Port, Community, SnmpVersion, AuthUser, AuthPassword, AuthProtocol, PrivPassword, PrivProtocol FROM snmphosts ORDER BY Id", function (err, rows) {
        if (err) {
            // LogSystemStatus(errorMessage, errorType, timeStamp, db, systemStatusManager)
            throw LogManager.error('Summit API: PUT /api/network/snmp/hosts - Error (Read)=' + err);
        }
        for (i = 0; i < rows.length; i++) {
            // Always send the raw value - clients responsible for conversion
            snmpHosts.push({
                "Id": rows[i].Id,
                "HostName": rows[i].HostName,
                "Enabled": rows[i].Enabled,
                "IpAddress": rows[i].IpAddress,
                "Port": rows[i].Port,
                "Community": rows[i].Community,
                "SnmpVersion": rows[i].SnmpVersion,
                "AuthUser": rows[i].AuthUser,
                "AuthPassword": rows[i].AuthPassword,
                "AuthProtocol": rows[i].AuthProtocol,
                "PrivPassword": rows[i].PrivPassword,
                "PrivProtocol": rows[i].PrivProtocol
            });
        }

        LogManager.debug('Summit API: PUT /api/network/snmp/hosts - Hosts=' + JSON.stringify(snmpHosts));
        UpdateSnmpManager(snmpHosts);
        res.json(snmpHosts);
    });
})

// Modify existing SNMP host
app.put('/api/network/snmp/hosts/:snmphost', function (req, res, next) {
    // Update the existing record
    LogManager.info('Summit API: PUT /api/network/snmp/hosts/' + req.params.snmphost);

    var update = "UPDATE SnmpHosts SET HostName='" + req.body.HostName +
        "', Enabled='" + req.body.Enabled +
        "', IpAddress='" + req.body.IpAddress +
        "', Port='" + req.body.Port +
        "', Community='" + req.body.Community +
        "', SnmpVersion='" + req.body.SnmpVersion +
        "', AuthUser='" + req.body.AuthUser +
        "', AuthPassword='" + req.body.AuthPassword +
        "', AuthProtocol=" + req.body.AuthProtocol +
        ", PrivPassword='" + req.body.PrivPassword +
        "', PrivProtocol=" + req.body.PrivProtocol +
        " WHERE rowid='" + req.params.snmphost + "'";
    SqliteManager.run(update, function (err) {
        if (err !== null) {
            res.send({ msg: "SQLite3 dB: api/snmphosts/" + req.body.Id + " UPDATE err=" + JSON.stringify(err) })
            throw LogManager.error('Summit API: PUT /api/network/snmp/hosts/' + req.params.snmphost + ' - Error (Update)=' + JSON.stringify(err));
        }
    });

    // Get the complete list of SNMP Hosts and update I/O processing sub-systems
    var snmpHosts = [];
    SqliteManager.all("SELECT rowid as Id, HostName, Enabled, IpAddress, Port, Community, SnmpVersion, AuthUser, AuthPassword, AuthProtocol, PrivPassword, PrivProtocol FROM snmphosts ORDER BY Id", function (err, rows) {
        if (err) {
            throw LogManager.error('Summit API: PUT /api/network/snmp/hosts/' + req.params.snmphost + ' - Error (Read)=' + err);
        }
        for (i = 0; i < rows.length; i++) {
            // Always send the raw value - clients responsible for conversion
            snmpHosts.push({
                "Id": rows[i].Id,
                "HostName": rows[i].HostName,
                "Enabled": rows[i].Enabled,
                "IpAddress": rows[i].IpAddress,
                "Port": rows[i].Port,
                "Community": rows[i].Community,
                "SnmpVersion": rows[i].SnmpVersion,
                "AuthUser": rows[i].AuthUser,
                "AuthPassword": rows[i].AuthPassword,
                "AuthProtocol": rows[i].AuthProtocol,
                "PrivPassword": rows[i].PrivPassword,
                "PrivProtocol": rows[i].PrivProtocol
            });

            if (rows[i].Id == req.params.snmphost) {
                snmpHostReturn = rows[i];
            }
        }
        LogManager.debug('Summit API: PUT /api/network/snmp/hosts/' + req.params.snmphost + ' - Hosts=' + JSON.stringify(snmpHosts));
        UpdateSnmpManager(snmpHosts);
        res.send(snmpHostReturn);
    });
})

function UpdateSnmpManager(snmpHosts) {
    SnmpManager = snmpHosts;
    DigitalInputsManager.UpdateSnmpManager(SnmpManager);
    DigitalOutputsManager.UpdateSnmpManager(SnmpManager);
    AnalogInputsManager.UpdateSnmpManager(SnmpManager);
    AnalogOutputsManager.UpdateSnmpManager(SnmpManager);
    SerialPortsManager.UpdateSnmpManager(SnmpManager);
    PingSensorManager.UpdateSnmpManager(SnmpManager);
}

// Test a SNMP host by sending a trap message
app.put('/api/network/snmp/test', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/network/snmp/test');

    LogManager.debug('Summit API: PUT /api/network/snmp/test - Host=' + JSON.stringify(req.body));

    // Define the notification
    var snmpCommand;
    var snmpHost;

    var snmpNotification = "";
    snmpNotification += "1.3.6.1.4.1.11902.10.5.1 ";
    snmpNotification += "1.3.6.1.4.1.11902.10.5.2 s 'Test Trap'";

    // Set up the SNMP trap command
    snmpCommand = "snmptrap ";
    if (req.body.SnmpVersion == 2) {
        snmpCommand += "-v 2c ";
        snmpCommand += "-c " + req.body.Community + " ";
    }
    else if (req.body.SnmpVersion == 3) {
        snmpCommand += "-v 3 ";
        var EngineId = "0x" + req.body.Community.replace(/\-/g, '');
        snmpCommand += "-e " + EngineId + " ";
        snmpCommand += "-u " + req.body.AuthUser + " ";
        if (req.body.AuthProtocol > 0) snmpCommand += "-a " + AuthProtocol[req.body.AuthProtocol] + " -A " + req.body.AuthPassword + " ";
        if (req.body.PrivProtocol > 0) snmpCommand += "-x " + PrivProtocol[req.body.PrivProtocol] + " -X " + req.body.PrivPassword + " ";
        var securityLevel = "-l noAuthNoPriv ";
        if (req.body.AuthProtocol > 0 && req.body.PrivProtocol == 0)
            securityLevel = "-l authNoPriv ";
        if (req.body.AuthProtocol > 0 && req.body.PrivProtocol > 0)
            securityLevel = "-l authPriv ";
        snmpCommand += securityLevel;
    } 

    snmpHost = req.body.IpAddress + " '' ";

    cp.exec(snmpCommand + snmpHost + snmpNotification, function (error, stdout, stderr) {
        if (error !== null) {
            LogManager.error("Summit API: PUT /api/network/snmp/test - Error (Trap send):" + error);
        } else {
            LogManager.debug("Summit API: PUT /api/network/snmp/test. Success:" + stdout);
        }
        res.send(req.body);
    });
});

// Return the TASC-SUMMIT.MIB file
app.get('/api/network/snmp/mibfile', function (req, res, next) {
    LogManager.info('Summit API: GET /api/network/snmp/mibfile');

    // Read the TASC-SUMMIT.MIB file
    fs.readFile("./TASC-SUMMIT.MIB", 'utf8', function (err, data) {
        if (err) {
            LogManager.error('Summit API: GET /api/network/snmp/mibfile - Error (Read)=' + JSON.stringify(err));
            res.send("Error reading TASC-SUMMIT.MIB file");
        }
        else {
            LogManager.debug('Summit API: GET /api/network/snmp/mibfile - File=TASC-SUMMIT.MIB');
            res.send(encodeURIComponent(data));
        }
    });
})
//#endregion SNMP Support

//#endregion Networking, SMTP and SNMP API

//#region Logging API
//---------------------------------------------
// Logging API

app.get('/api/logging/loglevel', function (req, res, next) {
    LogManager.info('Summit API: GET /api/logging/loglevel');

    var loglevel;
    switch (LogManager.level.levelStr) {
        case "INFO":
            loglevel = "BASIC";
            break;

        case "DEBUG":
            loglevel = "VERBOSE";
            break;

        case "ALL":
            loglevel = "ALL";
            break;

    }
    LogManager.info('Summit API: PUT /api/logging/loglevel - Level=' + loglevel);
    res.json({ "LogLevel": loglevel });
});

app.put('/api/logging/loglevel/:loglevel', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/logging/loglevel/' + req.params.loglevel);

    switch (req.params.loglevel) {
        case "BASIC":
            LogManager.setLevel(log4js.levels.INFO);
            loglevel = "BASIC";
            LogManager.info("Logging Manager - Level=" + loglevel + " (" + LogManager.level.levelStr + ")");
            break;

        case "VERBOSE":
            LogManager.setLevel(log4js.levels.DEBUG);
            loglevel = "VERBOSE";
            LogManager.info("Logging Manager - Level=" + loglevel + " (" + LogManager.level.levelStr + ")");
            break;

        case "ALL":
            LogManager.setLevel(log4js.levels.ALL);
            loglevel = "ALL";
            LogManager.info("Logging Manager - Level=" + loglevel + " (" + LogManager.level.levelStr + ")");
            break;
    }
    LogManager.info('Summit API: PUT /api/logging/loglevel/' + req.params.loglevel + " - Level=" + LogManager.level.levelStr);
    res.json({ "LogLevel": loglevel });
});

// Get list of log files
app.get('/api/logging/logfiles', function (req, res, next) {
    LogManager.info('Summit API: GET /api/logging/logfiles');

    // Read the log files
    logDirectory = './log';
    var logFiles = [];
    fs.readdir(logDirectory, function (err, files) {
        if (err) {
            LogManager.error('Summit API: GET /api/logging/logfiles - Error (Read)=' + JSON.stringify(err));
            res.send('Error reading log files');
        }
        else {
            // Get a list of these files
            for (var i in files) {
                var filePath = logDirectory + '/' + files[i];
                try {
                    var stat = fs.statSync(filePath);
                    if (stat.isFile()) {
                        logFiles.push({
                            "Name": files[i],
                            "Size": parseInt(stat.size).toLocaleString(),
                            "Date": stat.mtime
                        });
                    }
                }
                catch (e) {
                    LogManager.error('Summit API: GET/api/logging/logfiles - Error reading file=' + e);
                }
            }

            // Put the files in the order that we want
            logFiles.sort().reverse(); // this will put 'summit.log' at end, but sort with most recent at front 
            var summitLog = logFiles.pop();// get 'summit.log'
            logFiles.unshift(summitLog);// put 'summit.log' in front

            LogManager.debug('Summit API: GET /api/logging/logfiles - Files=' + JSON.stringify(logFiles));
            res.send(logFiles);
        }
    });
});

// Read contents of a log file
app.get('/api/logging/logfiles/:logfile', function (req, res, next) {
    LogManager.info('Summit API: GET /api/logging/logfiles/' + req.params.logfile);

    // Read the log files
    logDirectory = './log';
    fs.readFile(logDirectory + "/" + req.params.logfile, 'utf8', function (err, data) {
        if (err) {
            LogManager.error('Summit API: GET /api/logging/logfiles/' + req.params.logfile + ' - Error (Read)=' + JSON.stringify(err));
            res.send("Error reading log file");
        }
        else {
            LogManager.debug('Summit API: GET /api/logging/logfiles - File=' + req.params.logfile);
            res.send(encodeURIComponent(data));
        }
    });
})
//#endregion Logging API

//#region Digital Inputs

// Get Digital Input points
app.get('/api/digitalinputs', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitalinputs');

    var _digitalInputs = [];
    SqliteManager.all("SELECT Address, Name, Value, Enabled, SendEvents, Polarity, HoldTime, TimeChanged FROM digitalinputs ORDER BY Address", function (err, rows) {
        if (err) {
            // LogSystemStatus(errorMessage, errorType, timeStamp, db, systemStatusManager)
            throw LogManager.error('Summit API: GET /api/digitalinputs - Error (Select)=' + JSON.stringify(err));
        }

        for (i = 0; i < rows.length; i++) {
            // Always send the raw value - clients responsible for conversion
            _digitalInputs.push({
                "Address": rows[i].Address,
                "Name": rows[i].Name,
                "Value": rows[i].Value,
                "Enabled": rows[i].Enabled,
                "SendEvents": rows[i].SendEvents,
                "Polarity": rows[i].Polarity,
                "HoldTime": rows[i].HoldTime,
                "TimeChanged": rows[i].TimeChanged
            });
        }
        LogManager.debug('Summit API: GET /api/digitalinputs - Points=' + JSON.stringify(_digitalInputs));
        res.json(_digitalInputs);
    });
})

// Set ALL Digital Inputs to disabled
app.get('/api/digitalinputs/all/disable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitalinputs/all/disable');

    for (var digitaloutput = 0; digitaloutput < DIGITAL_OUTPUTS_POINTS; digitaloutput++) {
        (function (digitaloutput) {
            var update = "UPDATE digitalinputs SET Enabled='FALSE' WHERE address='" + digitaloutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/digitalinputs/all/disable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(digitaloutput);
    }
    res.send(req.params)
})

// Set ALL Digital Inputs to enabled
app.get('/api/digitalinputs/all/enable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitalinputs/all/enable');

    for (var digitaloutput = 0; digitaloutput < DIGITAL_OUTPUTS_POINTS; digitaloutput++) {
        (function (digitaloutput) {
            var update = "UPDATE digitalinputs SET Enabled='TRUE' WHERE address='" + digitaloutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/digitalinputs/all/enable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(digitaloutput);
    }
    res.send(req.params)
})

// Get Digital Input point
app.get('/api/digitalinputs/:digitalinput', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitalinputs/' + req.params.digitalinput);

    var _digitalInput = [];
    SqliteManager.get("SELECT Address, Name, Value, Enabled, SendEvents, Polarity, HoldTime, TimeChanged FROM digitalinputs WHERE Address='" + req.params.digitalinput + "'", function (err, row) {
        if (err) {
            throw LogManager.error('Summit API: GET /api/digitalinputs - Error (Select)=' + JSON.stringify(err));
        }

        if (typeof row !== "undefined") {
            LogManager.info("RESTful GET  : Digital Inputs address=" + row.Address);
            // Always send the raw value - clients responsible for conversion
            _digitalInput.push({
                "Address": row.Address,
                "Name": row.Name,
                "Value": row.Value,
                "Enabled": row.Enabled,
                "SendEvents": row.SendEvents,
                "Polarity": row.Polarity,
                "HoldTime": row.HoldTime,
                "TimeChanged": row.TimeChanged
            });
            LogManager.info("RESTful GET  : Digital Input - Address=" + row.Address);
            res.json(_digitalInput);
        }
        else {
            throw LogManager.error('Summit API: GET /api/digitalinputs/' + req.params.digitalinput + ' - Error (Select)=' + JSON.stringify(err));
            res.json({ Error: "Undefined digitalinput" });
        }
    });
})

// Update Digital Input point
app.put('/api/digitalinputs/:digitalinput', function (req, res, next) {
    //  Name        (TEXT)      <device name>
    //  Enabled     (TEXT)      ”TRUE”,”FALSE”
    //  SendEvents  (TEXT)      ”TRUE”, “FALSE”
    //  Polarity    (TEXT)      ”Normally Closed”, “Normally Open”
    //  HoldTime    (INTEGER)   <0-255 seconds>
    LogManager.info('Summit API: PUT /api/digitalinputs/' + req.params.digitalinput);

    var update = "UPDATE digitalinputs SET Name='" + req.body.Name + "', Enabled='" + req.body.Enabled + "', SendEvents='" + req.body.SendEvents + "', Polarity='" + req.body.Polarity + "', HoldTime='" + req.body.HoldTime + "' WHERE address='" + req.params.digitalinput + "'";
    SqliteManager.run(update, function (err) {
        if (err !== null) {
            LogManager.error('Summit API: PUT /api/digitalinputs/' + req.params.digitalinput + ' - Error (Update)=' + JSON.stringify(err));
            res.send({ Error: 'Error updating digitalinput' })
        }
        else {
            // Modify the memory array
            DigitalInputsManager.UpdateDigitalInputConfiguration(req.params.digitalinput, req.body.Name, req.body.Value, req.body.Enabled, req.body.SendEvents, req.body.Polarity, req.body.HoldTime, req.body.TimeChanged);

            // Send back the req as results if everything OK
            LogManager.debug('Summit API: PUT /api/digitalinputs/' + req.params.digitalinput + ' - Point=' + JSON.stringify(req.body));
            res.send(JSON.stringify(req.body))
        }
    });
})

//#endregion Digital Inputs

//#region Digital Outputs

// Get Digital Ouput points
app.get('/api/digitaloutputs', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitaloutputs');

    var _digitalOutputs = [];
    SqliteManager.all("SELECT Address, Name, Value, Enabled, SendEvents, Polarity, ActionType, TimeChanged FROM digitaloutputs ORDER BY Address", function (err, rows) {
        if (err) {
            throw LogManager.error('Summit API: GET /api/digitaloutputs - Error (Select)=' + JSON.stringify(err));
        }

        // Always send the raw value - clients responsible for conversion
        for (i = 0; i < rows.length; i++) {
            _digitalOutputs.push({
                "Address": rows[i].Address,
                "Name": rows[i].Name,
                "Value": rows[i].Value,
                "Enabled": rows[i].Enabled,
                "SendEvents": rows[i].SendEvents,
                "Polarity": rows[i].Polarity,
                "ActionType": rows[i].ActionType,
                "TimeChanged": rows[i].TimeChanged
            });
        }
        LogManager.debug('Summit API: GET /api/digitaloutputs - Points=' + JSON.stringify(_digitalOutputs));
        res.json(_digitalOutputs);
    });
})

// Set ALL Digital Outputs to disabled
app.get('/api/digitaloutputs/all/disable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitaloutputs/all/disable');

    for (var digitaloutput = 0; digitaloutput < DIGITAL_OUTPUTS_POINTS; digitaloutput++) {
        (function (digitaloutput) {
            var update = "UPDATE digitaloutputs SET Enabled='FALSE' WHERE address='" + digitaloutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/digitaloutputs/all/disable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(digitaloutput);
    }
    res.send(req.params)
})

// Set ALL Digital Outputs to enabled
app.get('/api/digitaloutputs/all/enable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitaloutputs/all/enable');

    for (var digitaloutput = 0; digitaloutput < DIGITAL_OUTPUTS_POINTS; digitaloutput++) {
        (function (digitaloutput) {
            var update = "UPDATE digitaloutputs SET Enabled='TRUE' WHERE address='" + digitaloutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/digitaloutputs/all/enable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(digitaloutput);
    }
    res.send(req.params)
})

// Set ALL Digital Outputs a point value
app.get('/api/digitaloutputs/all/:value', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/digitaloutputs/all/' + req.params.value);

    for (var digitaloutput = 0; digitaloutput < DIGITAL_OUTPUTS_POINTS; digitaloutput++) {
        (function (digitaloutput, setvalue) {
            var value;
            var polarity;
            // Get the digital output settings to determine if this is NC or NO
            SqliteManager.get("SELECT Polarity FROM digitaloutputs WHERE Address='" + digitaloutput + "'", function (err, row) {
                if (typeof row !== "undefined") {
                    // Ensure we know the proper polarity
                    polarity = row.Polarity;
                }

                // Determine the value to set
                var index = DIGITAL_OUTPUTS_BASE + parseInt(digitaloutput);
                if (polarity === "Normally Open") {
                    if (setvalue == "0") {
                        value = "0";
                    }
                    else {
                        value = "1";
                    }
                }
                else {
                    if (setvalue == "0") {
                        value = "1";
                    }
                    else {
                        value = "0";
                    }
                }

                // Set the value ...
                fs.writeFile("/sys/class/gpio/gpio" + index + "/value", value, function (err) {
                    if (err) {
                        LogManager.error('Summit API: PUT /api/digitaloutputs/all/' + setvalue + ' - Error (Write)=' + JSON.stringify(err));
                    }
                });
            });
        })(digitaloutput, req.params.value);
    }
    res.send(req.params)
})

// Get Digital Output point
app.get('/api/digitaloutputs/:digitaloutput', function (req, res, next) {
    LogManager.info('Summit API: GET /api/digitaloutputs/' + req.params.digitaloutput);

    var _digitalOutput = [];
    SqliteManager.get("SELECT Address, Name, Value, Enabled, SendEvents, Polarity, ActionType, TimeChanged FROM digitaloutputs WHERE Address='" + req.params.digitaloutput + "'", function (err, row) {
        if (err) {
            throw LogManager.error('Summit API: GET /api/digitaloutputs/' + req.params.digitaloutput + ' - Error (Select)=' + JSON.stringify(err));
        }

        if (typeof row !== "undefined") {
            // Always send the raw value - clients responsible for conversion
            _digitalOutput.push({
                "Address": row.Address,
                "Name": row.Name,
                "Value": row.Value,
                "Enabled": row.Enabled,
                "SendEvents": row.SendEvents,
                "Polarity": row.Polarity,
                "ActionType": row.ActionType,
                "TimeChanged": row.TimeChanged
            });
            LogManager.debug('Summit API: GET /api/digitaloutputs/' + req.params.digitaloutput + ' - Point=' + JSON.stringify(_digitalOutput));
            res.json(_digitalOutput);
        }
        else {
            LogManager.error('Summit API: GET /api/digitaloutputs/' + req.params.digitaloutput + ' - Error (Select)=' + JSON.stringify(err));
            res.json({ Error: "Undefined digitaloutput" });
        }
    });
})

// Update Digital Output point
app.put('/api/digitaloutputs/:digitaloutput', function (req, res, next) {
    //  Name        (TEXT)      <device name>
    //  Enabled     (TEXT)      ”TRUE”,”FALSE”
    //  SendEvents  (TEXT)      ”TRUE”, “FALSE”
    //  Polarity    (TEXT)      ”Normally Closed”, “Normally Open”
    //  ActionType  (TEXT)      "Latched", "Pulse"
    LogManager.info('Summit API: PUT /api/digitaloutputs/' + req.params.digitaloutput);

    var update = "UPDATE digitaloutputs SET Name='" + req.body.Name + "', Enabled='" + req.body.Enabled + "', SendEvents='" + req.body.SendEvents + "', Polarity='" + req.body.Polarity + "', ActionType='" + req.body.ActionType + "' WHERE address='" + req.params.digitaloutput + "'";
    SqliteManager.run(update, function (err) {
        if (err !== null) {
            LogManager.error('Summit API: PUT /api/digitaloutputs/' + req.params.digitaloutput + ' - Error (Update)=' + JSON.stringify(err));
            res.send({ Error: 'Error updating digitaloutput' })
        }
        else {
            // Modify the memory array
            DigitalOutputsManager.UpdateDigitalOutputConfiguration(req.params.digitaloutput, req.body.Name, req.body.Value, req.body.Enabled, req.body.SendEvents, req.body.Polarity, req.body.ActionType, req.body.TimeChanged)

            // Send back the req as results if everything OK
            LogManager.debug('Summit API: PUT /api/digitaloutputs/' + req.params.digitaloutput + ' - Point=' + JSON.stringify(req.body));
            res.send(JSON.stringify(req.body))
        }
    });
})

// Set Digital Output point value
app.put('/api/digitaloutputs/:digitaloutput/:value', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/digitaloutputs/' + req.params.digitaloutput + '/' + req.params.value);

    // Get the digital output settings to determine if this is NC or NO
    SqliteManager.get("SELECT Polarity FROM digitaloutputs WHERE Address='" + req.params.digitaloutput + "'", function (err, row) {
        if (typeof row !== "undefined") {
            // Ensure we know the proper polarity
            polarity = row.Polarity;
        }

        // Determine the value to set
        var index = DIGITAL_OUTPUTS_BASE + parseInt(req.params.digitaloutput);
        if (polarity === "Normally Open") {
            if (req.params.value == "0") {
                value = "0";
            }
            else {
                value = "1";
            }
        }
        else {
            if (req.params.value == "0") {
                value = "1";
            }
            else {
                value = "0";
            }
        }

        // Set the value ...
        var index = DIGITAL_OUTPUTS_BASE + parseInt(req.params.digitaloutput);
        fs.writeFile("/sys/class/gpio/gpio" + index + "/value", value, function (err) {
            if (err) {
                LogManager.error('Summit API: PUT /api/digitaloutputs/' + req.params.digitaloutput + '/' + req.params.value + ' - Error (Write)=' + JSON.stringify(err));
                res.send({ Error: 'Error setting digitaloutput' })
            }

            LogManager.debug('Summit API: PUT /api/digitaloutputs/' + req.params.digitaloutput + '/' + req.params.value + ' - Point=' + JSON.stringify(req.params));
            res.send(req.params)
        });
    });
})

//#endregion Digital Outputs

//#region Analog Inputs

// Get Analog Input points
app.get('/api/analoginputs', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analoginputs');

    var _analogInputs = [];
    SqliteManager.all("SELECT Address, Name, FormulaId, FormulaEnabled, RawValue, RangedValue, ThresholdState, Enabled, SendEvents, Range, LowThreshold, HighThreshold, Offset, HoldTime, Qualifier, TimeChanged FROM analoginputs ORDER BY Address", function (err, rows) {
        if (err) {
            throw LogManager.error('Summit API: GET /api/analoginputs - Error (Select)=' + JSON.stringify(err));
        }

        for (i = 0; i < rows.length; i++) {
            _analogInputs.push({
                "Address": rows[i].Address,
                "Name": rows[i].Name,
                "FormulaId": rows[i].FormulaId,
                "FormulaEnabled": rows[i].FormulaEnabled,
                "RawValue": rows[i].RawValue,
                "FormulaValue": AnalogInputsManager.GetFormulaValue(i),
                "Units": AnalogInputsManager.GetAnalogInputUnits(i),
                "TempValue": rows[i].RawValue,
                "RangedValue": rows[i].RangedValue,
                "ThresholdState": rows[i].ThresholdState,
                "Enabled": rows[i].Enabled,
                "SendEvents": rows[i].SendEvents,
                "Range": rows[i].Range,
                "LowThreshold": rows[i].LowThreshold,
                "HighThreshold": rows[i].HighThreshold,
                "FormulaLowThreshold": AnalogInputsManager.GetFormulaLowThreshold(i),
                "FormulaHighThreshold": AnalogInputsManager.GetFormulaHighThreshold(i),
                "Offset": rows[i].Offset,
                "HoldTime": rows[i].HoldTime,
                "Qualifier": rows[i].Qualifier,
                "QualifierValue": AnalogInputsManager.GetAnalogInputQualifierValue(i),
                "TimeChanged": rows[i].TimeChanged
            });
        }
        LogManager.debug('Summit API: GET /api/analoginputs - Points=' + JSON.stringify(_analogInputs));
        res.json(_analogInputs);
    });
})

app.get('/api/analoginputs/realtime', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analoginputs/realtime');

    var _analogInputs = [];
    _analogInputs = AnalogInputsManager.GetAnalogInputsRealtime();
    res.json(_analogInputs);
})

// Set ALL Analog Inputs to disabled
app.get('/api/analoginputs/all/disable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analoginputs/all/disable');

    for (var analogoutput = 0; analogoutput < DIGITAL_OUTPUTS_POINTS; analogoutput++) {
        (function (analogoutput) {
            var update = "UPDATE analoginputs SET Enabled='FALSE' WHERE address='" + analogoutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/analoginputs/all/disable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(analogoutput);
    }
    res.send(req.params)
})

// Set ALL Analog Inputs to enabled
app.get('/api/analoginputs/all/enable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analoginputs/all/enable');

    for (var analogoutput = 0; analogoutput < DIGITAL_OUTPUTS_POINTS; analogoutput++) {
        (function (analogoutput) {
            var update = "UPDATE analoginputs SET Enabled='TRUE' WHERE address='" + analogoutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/analoginputs/all/enable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(analogoutput);
    }
    res.send(req.params)
})

// Get Analog Input point
app.get('/api/analoginputs/:analoginput', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analoginputs/' + req.params.analoginput);

    var _analogInput = [];
    SqliteManager.get("SELECT Address, Name, FormulaId, FormulaEnabled, RawValue, RangedValue, ThresholdState, Enabled, SendEvents, Range, LowThreshold, HighThreshold, Offset, HoldTime, Qualifier, TimeChanged FROM analoginputs WHERE Address='" + req.params.analoginput + "'", function (err, row) {
        if (err) {
            throw LogManager.error('Summit API: GET /api/analoginputs/' + req.params.analoginput + ' - Error (Select)=' + JSON.stringify(err));
        }

        if (typeof row !== "undefined") {
            _analogInput.push({
                "Address": row.Address,
                "Name": row.Name,
                "FormulaId": row.FormulaId,
                "FormulaEnabled": row.FormulaEnabled,
                "RawValue": row.RawValue,
                "RangedValue": row.RangedValue,
                "FormulaValue": AnalogInputsManager.GetFormulaValue(row.Address),
                "Units": AnalogInputsManager.GetAnalogInputUnits(row.Address),
                "ThresholdState": row.ThresholdState,
                "Enabled": row.Enabled,
                "SendEvents": row.SendEvents,
                "Range": row.Range,
                "LowThreshold": row.LowThreshold,
                "HighThreshold": row.HighThreshold,
                "FormulaLowThreshold": AnalogInputsManager.GetFormulaLowThreshold(row.Address),
                "FormulaHighThreshold": AnalogInputsManager.GetFormulaHighThreshold(row.Address),
                "Offset": row.Offset,
                "HoldTime": row.HoldTime,
                "Qualifier": row.Qualifier,
                "QualifierValue": AnalogInputsManager.GetAnalogInputQualifierValue(row.Address),
                "TimeChanged": row.TimeChanged
            });
            LogManager.debug('Summit API: GET /api/analoginputs/' + req.params.analoginput + ' - Points=' + JSON.stringify(_analogInput));
            res.json(_analogInput);
        }
        else {
            res.json({ Error: "Undefined analoginput" });
            throw LogManager.error('Summit API: GET /api/analoginputs/' + req.params.analoginput + ' - Error (Select)=' + JSON.stringify(err));
        }
    });
})

// Update Analog Input point
app.put('/api/analoginputs/:analoginput', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/analoginputs/' + req.params.analoginput + " data=" + JSON.stringify(req.body));

    var update = "UPDATE analoginputs SET Name='" + req.body.Name + "', FormulaId='" + req.body.FormulaId + "', FormulaEnabled='" + req.body.FormulaEnabled + "', Enabled='" + req.body.Enabled + "', SendEvents='" + req.body.SendEvents + "', Range='" + req.body.Range + "', LowThreshold='" + req.body.LowThreshold + "', HighThreshold='" + req.body.HighThreshold + "', Offset='" + req.body.Offset + "', Qualifier='" + req.body.Qualifier + "', HoldTime='" + req.body.HoldTime + "' WHERE address='" + req.params.analoginput + "'";
    SqliteManager.run(update, function (err) {
        if (err !== null) {
            LogManager.error('Summit API: PUT /api/analoginputs/' + req.params.analoginput + ' - Error (Update)=' + JSON.stringify(err));
            res.send({ Error: 'Error updating analoginput' })
        }
        else {
            // Modify the memory array
            AnalogInputsManager.UpdateAnalogInputConfiguration(req.params.analoginput, req.body.Name, req.body.FormulaId, req.body.FormulaEnabled, req.body.RawValue, req.body.RangedValue, req.body.FormulaValue, req.body.ThresholdState, req.body.Enabled, req.body.SendEvents, req.body.Range, req.body.LowThreshold, req.body.HighThreshold, req.body.FormulaLowThreshold, req.body.FormulaHighThreshold, req.body.Offset, req.body.HoldTime, req.body.Qualifier, req.body.TimeChanged);
            AnalogInputsManager.ForceNewRawValue(req.params.analoginput);// Force a read, in case something has changed, for example, new thresholds

            // Send back the req as results if everything OK
            var AnalogInput = AnalogInputsManager.GetAnalogInputValues(req.params.analoginput);
            LogManager.debug('Summit API: PUT /api/analoginputs/' + req.params.analoginput + ' - Point=' + JSON.stringify(AnalogInput));
            res.send(JSON.stringify(AnalogInput))
        }
    });
})

//#endregion Analog Inputs

//#region Formulas

// Get Formulas Array
app.get('/api/formulas', function (req, res, next) {
    LogManager.info('Summit API: GET /api/formulas');

    // Get Formulas Array
    AnalogInputsManager.GetFormulas(function (FormulasArray) {
        LogManager.debug('Summit API: GET /api/formulas - Formulas=' + JSON.stringify(FormulasArray));
        res.json(FormulasArray);
    });
});

// Add Formulas
app.put('/api/formulas', function (req, res, next) {
    //  Name        (TEXT)      <name>
    //  Enabled     (TEXT)      <formula>
    //  Code        (TEXT)      <units>
    //  Decimals    (INTEGER)   <decimals>
    //  TimeChanged (TEXT)      <time changed>
    LogManager.debug('Summit API: PUT /api/formulas - Formula=' + JSON.stringify(req.body));

    // Add Formula via req parameters
    AnalogInputsManager.AddFormula(req, function (FormulasArray) {
        LogManager.debug('Summit API: PUT /api/formulas  - Formulas=' + JSON.stringify(FormulasArray));
        res.json(FormulasArray);
    });
});

// Modify existing formula
app.put('/api/formulas/evaluate', function (req, res, next) {
    //  Id          (TEXT)      <Id of the host>
    //  Name        (TEXT)      <name>
    //  Enabled     (TEXT)      <formula>
    //  Code        (TEXT)      <units>
    //  Decimals    (INTEGER)   <decimals>
    //  TimeChanged (TEXT)      <time changed>
    LogManager.debug('Summit API: PUT /api/formulas/evaluate - Formula=' + JSON.stringify(req.body));

    // Evaluate an existing Formula via req params
    AnalogInputsManager.EvaluateFormula(req.body, function (result) {
        LogManager.debug('Summit API: PUT /api/formulas/evaluate - result=' + JSON.stringify(result));
        res.json(result);
    });
})

// Validate existing formula
app.put('/api/formulas/validate', function (req, res, next) {
    //  Id          (TEXT)      <Id of the host>
    //  Name        (TEXT)      <name>
    //  Enabled     (TEXT)      <formula>
    //  Code        (TEXT)      <units>
    //  Decimals    (INTEGER)   <decimals>
    //  TimeChanged (TEXT)      <time changed>
    LogManager.debug('Summit API: PUT /api/formulas/validate - Formula=' + JSON.stringify(req.body));

    // Validate an existing Formula via req params
    AnalogInputsManager.ValidateFormula(req, function (result) {
        LogManager.debug('Summit API: PUT /api/formulas/validate - result=' + JSON.stringify(result));
        res.json(result);// true if parsed, false if parsing error
    });
})

// Delete a formula
app.put('/api/formulas/delete', function (req, res, next) {
    //  Id          (TEXT)      <Id of the host>
    //  Name        (TEXT)      <name>
    //  Enabled     (TEXT)      <formula>
    //  Code        (TEXT)      <units>
    //  Decimals    (INTEGER)   <decimals>
    //  TimeChanged (TEXT)      <time changed>
    LogManager.debug('Summit API: PUT /api/formulas/delete - Formula=' + JSON.stringify(req.body));

    // Delete an existing Formula via req params
    AnalogInputsManager.DeleteFormula(req, function (FormulasArray) {
        LogManager.debug('Summit API: PUT /api/formulas  - Formulas=' + JSON.stringify(FormulasArray));
        res.json(FormulasArray);
    });

})

// Modify existing formula
app.put('/api/formulas/:formula', function (req, res, next) {
    //  Id          (TEXT)      <Id of the host>
    //  Name        (TEXT)      <name>
    //  Enabled     (TEXT)      <formula>
    //  Code        (TEXT)      <units>
    //  Decimals    (INTEGER)   <decimals>
    //  TimeChanged (TEXT)      <time changed>
    LogManager.info('Summit API: PUT /api/formulas/:' + req.params.formula + " Formula=" + JSON.stringify(req.body));

    // Modify an existing Formula via req params
    AnalogInputsManager.ModifyFormula(req, function (FormulasArray) {
        LogManager.debug('Summit API: PUT /api/formulas/:' + FormulasArray.Id + ' - Formulas=' + JSON.stringify(FormulasArray));
        res.json(FormulasArray);
    });
})

//#endregion Formulas

//#region Analog Outputs

// Get Analog Output points
app.get('/api/analogoutputs', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analogoutputs');

    var _analogOutputs = [];
    SqliteManager.all("SELECT Address, Name, RawValue, RangedValue, Enabled, SendEvents, Range, TimeChanged FROM analogoutputs ORDER BY Address", function (err, rows) {
        if (err) {
            throw LogManager.error('Summit API: GET /api/analogoutputs - Error (Select)=' + JSON.stringify(err));
        }

        for (i = 0; i < rows.length; i++) {
            _analogOutputs.push({
                "Address": rows[i].Address,
                "Name": rows[i].Name,
                "RawValue": rows[i].RawValue,
                "RangedValue": rows[i].RangedValue,
                "Enabled": rows[i].Enabled,
                "SendEvents": rows[i].SendEvents,
                "Range": rows[i].Range,
                "TimeChanged": rows[i].TimeChanged
            });
        }
        LogManager.debug('Summit API: GET /api/analogoutputs - Points=' + JSON.stringify(_analogOutputs));
        res.json(_analogOutputs);
    });
})

// Set ALL Analog Outputs to disabled
app.get('/api/analogoutputs/all/disable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analogoutputs/all/disable');

    for (var analogoutput = 0; analogoutput < DIGITAL_OUTPUTS_POINTS; analogoutput++) {
        (function (analogoutput) {
            var update = "UPDATE analogoutputs SET Enabled='FALSE' WHERE address='" + analogoutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/analogoutputs/all/disable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(analogoutput);
    }
    res.send(req.params)
})

// Set ALL Analog Outputs to enabled
app.get('/api/analogoutputs/all/enable', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analogoutputs/all/enable');

    for (var analogoutput = 0; analogoutput < DIGITAL_OUTPUTS_POINTS; analogoutput++) {
        (function (analogoutput) {
            var update = "UPDATE analogoutputs SET Enabled='TRUE' WHERE address='" + analogoutput + "'";
            SqliteManager.run(update, function (err) {
                if (err !== null) {
                    LogManager.error('Summit API: GET /api/analogoutputs/all/enable - Error (Update)=' + JSON.stringify(err));
                }
            });
        })(analogoutput);
    }
    res.send(req.params)
})

// Get Analog Output point
app.get('/api/analogoutputs/:analogoutput', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analogoutputs/' + req.params.analogoutput);

    var _analogOutput = [];
    SqliteManager.get("SELECT Address, Name, RawValue, RangedValue, Enabled, SendEvents, Range, TimeChanged FROM analogoutputs WHERE Address='" + req.params.analogoutput + "'", function (err, row) {
        if (err) {
            throw LogManager.error('Summit API: GET /api/analogoutputs/' + req.params.analogoutput + ' - Error (Select)=' + JSON.stringify(err));
        }

        if (typeof row !== "undefined") {
            _analogOutput.push({
                "Address": row.Address,
                "Name": row.Name,
                "RawValue": row.RawValue,
                "RangedValue": row.RangedValue,
                "Enabled": row.Enabled,
                "SendEvents": row.SendEvents,
                "Range": row.Range,
                "TimeChanged": row.TimeChanged
            });
            LogManager.debug('Summit API: GET /api/analogoutputs/' + req.params.analogoutput + ' - Point=' + JSON.stringify(_analogOutput));
            res.json(_analogOutput)
        }
        else {
            LogManager.error('Summit API: GET /api/analogoutputs/' + req.params.analogoutput + ' - Error (Select)=' + JSON.stringify(err));
            res.json({ Error: "Undefined analogoutput" });
        }
    });
})

app.get('/api/analogoutputs/all/:value', function (req, res, next) {
    LogManager.info('Summit API: GET /api/analogoutputs/all/' + req.params.value);

    if (req.params.value) {
        //Get range from the database, so we know what value to write out to the file
        var analogOutputRangedValue = req.params.value;
        var analogOutputRawValue = 0;

        for (var analogOutput = 0; analogOutput < ANALOG_OUTPUTS_POINTS; analogOutput++) {
            (function (analogOutput, analogOutputRangedValue, analogOutputRawValue) {
                SqliteManager.get("SELECT Range FROM AnalogOutputs WHERE Address='" + analogOutput + "'", function (err, row) {
                    if (err) {
                        LogManager.error('Summit API: GET /api/analogoutputs/all/' + analogOutputRangedValue + ' - Error (Select)=' + JSON.stringify(err));
                        res.send({ Error: 'Error setting analogoutput' })
                    }

                    if (typeof row !== "undefined") {
                        var analogOutputRange = row.Range;
                        if (Number(analogOutputRangedValue) > Number(analogOutputRange)) analogOutputRangedValue = analogOutputRange;
                        analogOutputRawValue = parseInt((analogOutputRangedValue * ANALOG_OUTPUT_CONVERT_TO_RAW_SLOPE) + ANALOG_OUTPUT_CONVERT_TO_RAW_OFFSET);
                        if (analogOutputRawValue < 0) analogOutputRawValue = 0;
                        LogManager.info('Summit API: GET /api/analogoutputs/all/' + analogOutputRangedValue + " Converted Raw Value=" + analogOutputRawValue);

                        fs.writeFile("/sys/class/hwmon/hwmon2/out" + analogOutput + "_output", analogOutputRawValue, function (err) {
                            if (err) {
                                LogManager.error('Summit API: GET /api/analogoutputs/all/' + analogOutputRangedValue + ' - Error (Write)=' + JSON.stringify(err));
                            }
                        });
                    }
                });
            })(analogOutput, analogOutputRangedValue, analogOutputRawValue);
        }
    }
    else {
        LogManager.error('Summit API: GET /api/analogoutputs/all/' + req.params.value + ' - Error (Value)');
        res.send({ Error: 'Error analogoutput value' })
    }
    res.send(req.params)
})

// Update Analog Output point
app.put('/api/analogoutputs/:analogoutput', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput);

    var update = "UPDATE analogoutputs SET Name='" + req.body.Name + "', Enabled='" + req.body.Enabled + "', SendEvents='" + req.body.SendEvents + "', Range='" + req.body.Range + "' WHERE address='" + req.params.analogoutput + "'";
    SqliteManager.run(update, function (err) {
        if (err !== null) {
            LogManager.error('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + ' - Error (Update)=' + JSON.stringify(err));
            res.send({ Error: 'Error updating analogoutput' })
        }
        else {
            if (req.body.Enabled == "FALSE") {
                var analogOutputRange = req.body.Range;
                var oldRangedValue = req.body.RangedValue;// Old Value
                req.body.RangedValue = "0.00";// Set to 0.00V because we are now disabled
                var analogOutput = parseInt(req.params.analogoutput);
                analogOutputRawValue = parseInt((req.body.RangedValue * ANALOG_OUTPUT_CONVERT_TO_RAW_SLOPE) + ANALOG_OUTPUT_CONVERT_TO_RAW_OFFSET);
                if (analogOutputRawValue < 0) analogOutputRawValue = 0;
                req.body.RawValue = analogOutputRawValue;
                LogManager.info('Summit API: PUT /api/analogoutputs/' + analogOutput + ' Previous RangedValue="' + oldRangedValue + '" Now Ranged Value="' + req.body.RangedValue + '" Converted Raw Value=' + analogOutputRawValue);

                fs.writeFile("/sys/class/hwmon/hwmon2/out" + analogOutput + "_output", analogOutputRawValue, function (err) {
                    if (err) {
                        LogManager.error('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + ' set RangedValue=0V - Error (Write)=' + JSON.stringify(err));
                        res.send({ Error: 'Error setting analogoutput' })
                    } else {
                        LogManager.debug('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + ' set RangedValue=0V - success');
                    }
                });

            }

            // Update the memory array
            AnalogOutputsManager.UpdateAnalogOutputConfiguration(req.params.analogoutput, req.body.Name, req.body.RawValue, req.body.RangedValue, req.body.Enabled, req.body.SendEvents, req.body.Range, req.body.HoldTime)

            // Send back the req as results if everything OK
            LogManager.debug('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + ' - Point=' + JSON.stringify(req.body));
            res.send(JSON.stringify(req.body))
        }
    });
})

// Set Analog Output point value
var ANALOG_OUTPUT_CONVERT_TO_RAW_SLOPE = 200;
var ANALOG_OUTPUT_CONVERT_TO_RAW_OFFSET = 0;
app.put('/api/analogoutputs/:analogoutput/:value', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + '/' + req.params.value);

    if (req.params.value) {
        //Get range from the database, so we know what value to write out to the file
        var analogOutput = parseInt(req.params.analogoutput);
        var analogOutputRangedValue = req.params.value;
        var analogOutputRawValue = 0;

        // Get analog output point information       
        SqliteManager.get("SELECT Range FROM AnalogOutputs WHERE Address='" + analogOutput + "'", function (err, row) {
            if (err) {
                LogManager.error('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + '/' + req.params.value + ' - Error (Select)=' + JSON.stringify(err));
                res.send({ Error: 'Error setting analogoutput' })
            }

            if (typeof row !== "undefined") {
                var analogOutputRange = row.Range;
                analogOutputRawValue = parseInt((analogOutputRangedValue * ANALOG_OUTPUT_CONVERT_TO_RAW_SLOPE) + ANALOG_OUTPUT_CONVERT_TO_RAW_OFFSET);
                if (analogOutputRawValue < 0) analogOutputRawValue = 0;
                LogManager.info('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + '/' + req.params.value + " Converted Raw Value=" + analogOutputRawValue);

                fs.writeFile("/sys/class/hwmon/hwmon2/out" + analogOutput + "_output", analogOutputRawValue, function (err) {
                    if (err) {
                        LogManager.error('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + '/' + req.params.value + ' - Error (Write)=' + JSON.stringify(err));
                        res.send({ Error: 'Error setting analogoutput' })
                    }
                });

                LogManager.debug('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + '/' + req.params.value + ' - Point=' + JSON.stringify(req.params));
                res.send(req.params)
            }
        });
    }
    else {
        LogManager.error('Summit API: PUT /api/analogoutputs/' + req.params.analogoutput + '/' + req.params.value + ' - Error (Value)');
        res.send({ Error: 'Error analogoutput value' })
    }
})

//#endregion Analog Outputs

//#region Serial Ports

//#region Device Options methods
// Get Device Options
app.get('/api/serialports/deviceoptions', function (req, res, next) {
    LogManager.info('Summit API: GET /api/serialports/deviceoptions');

    var serialPortDeviceOptions = SerialPortsManager.GetSerialPortDeviceOptions();
    LogManager.info('Summit API: GET /api/serialports/deviceoptions - DeviceOptions=' + JSON.stringify(serialPortDeviceOptions));
    res.send(JSON.stringify(serialPortDeviceOptions));
})
//#endregion Device Options methods

//#region Device Model methods
// Get Device Model
app.get('/api/serialports/devicemodel/:serialport', function (req, res, next) {
    LogManager.info('Summit API: GET /api/serialports/devicemodel/' + req.params.serialport);

    var deviceModel = SerialPortsManager.GetSerialPortDeviceModel(req.params.serialport);
    LogManager.info('Summit API: GET /api/serialports/devicemodel/' + req.params.serialport + ' - DeviceModel=' + JSON.stringify(deviceModel));
    res.send(JSON.stringify(deviceModel));
})

// Evaluate Formula
var Parser = require('expr-eval').Parser;// Library to manage parsing and evaluation of formulas
var mathparser = new Parser();
app.put('/api/serialports/devicemodel/formulas', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/serialports/devicemode/formulas');

    var _expression
    for (var i = 0; i < req.body.Formulas.length; i++) {
        _expression = mathparser.parse(req.body.Formulas[i].Formula);
        req.body.Formulas[i].FormulaLowThreshold = Number(_expression.evaluate({ x: req.body.Formulas[0].FormulaLowThreshold })).toFixed(req.body.Formulas[i].Decimals);
        req.body.Formulas[i].FormulaHighThreshold = Number(_expression.evaluate({ x: req.body.Formulas[0].FormulaHighThreshold })).toFixed(req.body.Formulas[i].Decimals);
        LogManager.info('Summit API: PUT /api/serialports/devicemode/formulas - Low[' + i + ']=' + req.body.Formulas[i].FormulaLowThreshold + ' High[' + i + ']=' + req.body.Formulas[i].FormulaHighThreshold);
    }
    res.send(JSON.stringify(req.body));
})

// Put Device Model
app.put('/api/serialports/devicemodel/:serialport', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/serialports/devicemodel/' + req.params.serialport);

    SerialPortsManager.UpdateSerialPortDeviceModel(req.params.serialport, req.body, function (devicemodel) {
        if (devicemodel === "Error") {
            LogManager.error('Summit API: PUT /api/serialports/devicemodel/' + req.params.serialport + ' - Error (Update)=' + JSON.stringify(err));
            res.send({ Error: 'Error updating serialports/devicemodel' + req.params.serialport });
        } else {
            LogManager.debug('Summit API: PUT /api/serialports/devicemodel/' + req.params.serialport + ' - Point=' + JSON.stringify(devicemodel));
            res.send(JSON.stringify(devicemodel));
        }
    });
})

//#endregion Device Model methods

//#region Serial Port methods
// Get Serial Ports
app.get('/api/serialports', function (req, res, next) {
    LogManager.info('Summit API: GET /api/serialports');

    var SerialPortSettings = [];
    SqliteManager.all("SELECT Address, Name, Enabled, SendEvents, Device, TimeChanged FROM serialports ORDER BY Address", function (error, rows) {
        if (error) {
            throw LogManager.error('Summit API: GET /api/serialports - Error (Select)=' + JSON.stringify(error));
        }

        for (i = 0; i < rows.length; i++) {
            SerialPortSettings.push({
                "Address": rows[i].Address,
                "Name": rows[i].Name,
                "Enabled": rows[i].Enabled,
                "SendEvents": rows[i].SendEvents,
                "Device": rows[i].Device,
                "TimeChanged": rows[i].TimeChanged
            });
        }
        LogManager.debug('Summit API: GET /api/serialports - Ports=' + JSON.stringify(SerialPortSettings));
        res.json(SerialPortSettings);
    });
})

// Get Serial Port x
app.get('/api/serialports/:serialport', function (req, res, next) {
    LogManager.info('Summit API: GET /api/serialports/' + req.params.serialport);

    var SerialPortSettings = [];
    SqliteManager.get("SELECT Address, Name, Enabled, SendEvents, Device, TimeChanged FROM serialports WHERE Address=" + req.params.serialport, function (error, row) {
        if (error) {
            throw LogManager.error('Summit API: GET /api/serialports/' + req.params.serialport + ' - Error (Select)=' + JSON.stringify(error));
        }

        LogManager.debug('Summit API: GET /api/serialports/' + req.params.serialport + ' - row=' + JSON.stringify(row));

        if (typeof row !== "undefined") {
            // Always send the raw value - clients responsible for conversion
            LogManager.info('Summit API: GET /api/serialports/' + req.params.serialport + ' - Ports=' + JSON.stringify(row));
            res.json(row);
        } else {
            LogManager.error('Summit API: GET /api/serialports/' + req.params.serialport + ' - No Data');
            res.json({ Error: 'Error retrieving serialport' });
        }
    });
})

// Update Serial Port 
app.put('/api/serialports/:serialport', function (req, res, next) {
    //  Name        (TEXT)      <device name>
    //  Enabled     (TEXT)      ”TRUE”,”FALSE”
    //  Device      (TEXT)      ”NONE", <DeviceName>
    LogManager.info('Summit API: PUT /api/serialports/' + req.params.serialport);

    SerialPortsManager.UpdateSerialPortSettingPort(req.body, function (serialport) {
        if (serialport === "Error") {
            LogManager.error('Summit API: PUT /api/serialports/' + req.params.serialport + ' - Error (Update)=' + JSON.stringify(err));
            res.send({ Error: 'Error updating serialports' });
        } else {
            LogManager.debug('Summit API: PUT /api/serialports/' + req.params.serialport + ' - Point=' + JSON.stringify(serialport));
            res.send(JSON.stringify(serialport));
        }
    });
})

// Reset Serial Port x
app.put('/api/serialports/reset/:serialport', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/serialports/reset/' + req.params.serialport);

    SerialPortsManager.ResetSerialPortSettingPort(req.params.serialport, function (result) {
        if (result > 0) {
            LogManager.error('Summit API: PUT /api/serialports/reset/' + req.params.serialport + ' - Error=' + JSON.stringify(result));
            res.send({ Status: 'ERROR' });
        } else {
            LogManager.info('Summit API: PUT /api/serialports/reset/' + req.params.serialport + ' - Success=' + result);
            res.send({ Status: "OK" });
        }
    });

})

// Zero Serial Port x
app.put('/api/serialports/zero/:serialport', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/serialports/zero/' + req.params.serialport);

    SerialPortsManager.ZeroSerialPortSettingPort(req.params.serialport, function (result) {
        if (result > 0) {
            LogManager.error('Summit API: PUT /api/serialports/zero/' + req.params.serialport + ' - Error=' + JSON.stringify(result));
            res.send({ Status: 'ERROR' });
        } else {
            LogManager.info('Summit API: PUT /api/serialports/zero/' + req.params.serialport + ' - Success=' + result);
            res.send({ Status: "OK" });
        }
    });

})
//#endregion Serial Port methods

//#endregion Serial Ports

//#region Ping Sensor Control

// Get Ping Sensor interval
app.get('/api/kenwood/pinginterval', function (req, res, next) {
    LogManager.info('Summit API: GET /api/kenwood/pinginterval');

    // Get the Ping Sensor interval
    LogManager.debug('Summit API: GET /api/kenwood/pinginterval - Ping Sensor Interval=' + JSON.stringify(PING_SENSOR_INTERVAL));
    res.json(PING_SENSOR_INTERVAL);
});

// Set Ping Sensor interval
app.put('/api/kenwood/pinginterval', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/kenwood/pinginterval');

    // Sets the Ping Sensor interval
    LogManager.debug('Summit API: PUT /api/kenwood/pinginterval - New ping interval=' + req.body.Interval);
    PING_SENSOR_INTERVAL = req.body.Interval;
    res.json({ Interval: PING_SENSOR_INTERVAL });
});

// Get Kenwood rules
app.get('/api/kenwood/rules', function (req, res, next) {
    LogManager.info('Summit API: GET /api/kenwood/rules');

    // Get the complete list of Kenwood rules
    PingSensorManager.GetKenwoodRules(function (pingSensorRules) {
        LogManager.debug('Summit API: GET /api/kenwood/rules - Kenwood Rules=' + JSON.stringify(pingSensorRules));
        res.json(pingSensorRules);
    });
});

// Create new Kenwood rule
app.put('/api/kenwood/rules', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/kenwood/rules');

    // Add a new Kenwood rule, 
    //  returns list of Kenwood rules( with new rule)
    PingSensorManager.AddKenwoodRule(req, function (pingSensorRules) {
        LogManager.debug('Summit API: PUT /api/kenwood/rules - Kenwood Rules=' + JSON.stringify(pingSensorRules));
        res.json(pingSensorRules);
    });
});


// Delete a Kenwood rule
app.put('/api/kenwood/rules/delete', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/kenwood/rules/delete');

    // Add a new Kenwood rule, 
    //  returns list of Kenwood rules( with new rule)
    PingSensorManager.DeleteKenwoodRule(req, function (pingSensorRules) {
        LogManager.debug('Summit API: PUT /api/kenwood/rules/delete - Kenwood Rules=' + JSON.stringify(pingSensorRules));
        res.json(pingSensorRules);
    });
});

// Modify a Kenwood rule
app.put('/api/kenwood/rules/:kenwoodrule', function (req, res, next) {
    LogManager.info('Summit API: PUT /api/kenwood/rules/' + req.params.kenwoodrule);

    // Modidy an existing Kenwood rule, 
    //  returns list of Kenwood rules( with edited rule)
    PingSensorManager.ModifyKenwoodRule(req, function (pingSensorRules) {
        LogManager.debug('Summit API: PUT /api/kenwood/rules=' + req.params.kenwoodrule + ' - Kenwood Rules=' + JSON.stringify(pingSensorRules));
        res.json(pingSensorRules);
    });
});
//#endregion Kenwood Control

//#region System Clock
//logger.info('Intializing System Clock')
//setInterval(sendTime, 1000)
//function sendTime() {
//    var currentTime = moment();
//    SocketIoManager.sockets.emit("CurrentSystemTime", { "CurrentSystemTime": currentTime });
//}

setInterval(function () {
    var NTPCheck = moment().diff(CrestStartTime, 'years');
    if (NTPCheck > 10) {
        CrestStartTime = moment();
        LogManager.warn("CrestUptime updated due to 10+ years difference between previous initial start time");
    }
    var Uptime = moment().diff(CrestStartTime, 'seconds');
    SocketIoManager.sockets.emit("CrestUptime", { "CrestUptime": Uptime });
}, 1000);
//#endregion System Clock
//#endregion Crest API Middleware 


